<?php declare(strict_types=1);

use EuKit\Base\Parts\NewsMLArticle;
use EuKit\Base\ViewModels\ArticlePage;
use Everyware\Plugin\PresentationPreview\Contracts\PreviewData;
use Infomaker\Everyware\Base\OpenContent\OpenContentProvider;
use Infomaker\Everyware\Base\OpenContent\QueryBuilder;
use Infomaker\Everyware\Support\Date;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\Twig\View;
use Infomaker\Imengine\Imengine;

if (isset($previewData) && $previewData instanceof PreviewData) {
    $article = new NewsMLArticle(new GenericPropertyObject());

    $article->fill([
        'body_raw' => $previewData->getContent(),
        'contentype' => 'Article',
        'pubdate' => Date::now()->toOcString(),
        'uuid' => $previewData->getUuid()
    ]);

    $imengineUri = $previewData->getImengineUri();

    if ( ! empty($imengineUri)) {
        Imengine::setup($imengineUri);
    }

    $relatedArticleUuids = $previewData->getRelatedArticles();
    $relatedConceptUuids = $previewData->getRelatedConcepts();

    $ocProvider = OpenContentProvider::setup();

    if ( ! empty($relatedArticleUuids)) {
        $ocProvider->setPropertyMap('Article');

        $articles = $ocProvider->queryWithRequirements([
            'q' => QueryBuilder::where('uuid', $relatedArticleUuids)->buildQueryString(),
            'contenttype' => 'Article',
            'limit' => count($relatedArticleUuids)
        ]);

        $article->set('articles', collect($articles));
    }
    if ( ! empty($relatedConceptUuids)) {
        $ocProvider->setPropertyMap('Concept');

        $concepts = $ocProvider->queryWithRequirements([
            'q' => QueryBuilder::where('uuid', $relatedConceptUuids)->buildQueryString(),
            'contenttype' => 'Concept',
            'limit' => count($relatedConceptUuids)
        ]);

        $groupedConcepts = collect($concepts)->groupBy('Type'); //EWB-2509

        $conceptMap = [
            'author' => 'authors',
            'category' => 'categories',
            'place' => 'places',
            'section' => 'sections',
            'story' => 'stories',
            'topic' => 'tags',
            'person' => 'tags',
            'organisation' => 'tags',
        ];

        foreach ($conceptMap as $type => $group) {
            if ($groupedConcepts->has($type)) {
                // Add group of concepts to existing list if present
                $article->set($group, collect($article->getMultiValue($group))->merge($groupedConcepts->get($type)));
            }
        }

        if ($groupedConcepts->has('sections')) {
            $article->set('section', $groupedConcepts->get('sections')->first()->get('name'));
        }
    }

    $articlePage = new ArticlePage($article);

    add_filter('body_class', static function($classes, $class){
        return array_merge(
            $classes,
            [   // Classes that WordPress typically add to a single article page.
                'article-template',
                'article-template-single-article',
                'article-template-single-article-php',
                'single',
                'single-article'
            ],
            [   // Class introduced by this theme.
                'article-preview'
            ]
        );
    }, 10, 2);

    View::render('@base/page/article-page', $articlePage->getViewData());
}
